@@ -6,10 +6,11 @@ from pprint import pprint
 from urllib import parse
 
 import click
+from googleapiclient.errors import HttpError
 
 from ytscraper.helper.configfile import update_config
-from ytscraper.helper.echo import echoe, echov
-from ytscraper.helper.export import export_to_csv, filter_text
+from ytscraper.helper.echo import echoe, echov, echow
+from ytscraper.helper.export import export_to_csv, export_to_sql, filter_text
 from ytscraper.helper.yt_api import (
     get_youtube_handle,
     related_search,
@@ -40,7 +41,9 @@ verbose = False
     type=click.IntRange(0, 100),
     help="Maximal number of recursion steps.",
 )
-@click.option("--api-key", "-k", type=str, help="API Key to use YouTube API v3.")
+@click.option(
+    "--api-key", "-k", type=str, help="API Key to use YouTube API v3."
+)
 @click.option(
     "--output-dir",
     "-o",
@@ -51,9 +54,15 @@ verbose = False
 @click.option(
     "--output-format",
     "-f",
-    type=click.Choice(["csv"]),
+    type=click.Choice(["csv", "sql"]),
     help="The file format of output files.",
 )
+@click.option(
+    "--output-name",
+    "-b",
+    type=str,
+    help="The name prefix of the output files.",
+)
 @click.option(
     "--region-code",
     "-r",
@@ -76,7 +85,14 @@ verbose = False
     "--encoding",
     "-e",
     type=click.Choice(["ascii", "utf-8", "smart"]),
-    help="Transform text to which encoding.",
+    help="Transforms text to another encoding.",
+)
+@click.option(
+    "--unique",
+    "-u",
+    is_flag=True,
+    default=False,
+    help="Do not process nodes again.",
 )
 @click.pass_context
 def search(context, search_type, query, **options):
@@ -94,9 +110,11 @@ def search(context, search_type, query, **options):
         "safe_search": "safeSearch",
     }
     api_options = {rename[key]: config[key] for key in rename if config[key]}
-    handle = get_handle(config["api_key"])
+    handle = get_handle(config["keys"])
 
-    start_videos = get_starter_videos(config, handle, api_options, search_type, query)
+    start_videos = get_starter_videos(
+        config, handle, api_options, search_type, query
+    )
     nodes = build_nodes(config, handle, api_options, start_videos)
     # Filter nodes
     for node in nodes:
@@ -105,8 +123,13 @@ def search(context, search_type, query, **options):
                 node[key] = filter_text(node[key], encoding=config["encoding"])
 
     if config["output_dir"] and config["output_format"] == "csv":
-        echov("Query finished! Start exporting files!", verbose)
-        export_to_csv(nodes, config["output_dir"])
+        echov("Query finished! Start exporting files to CSV!", verbose)
+        export_to_csv(nodes, config["output_dir"], config["output_name"])
+        echov(f"Exported results to: " + config["output_dir"])
+
+    if config["output_dir"] and config["output_format"] == "sql":
+        echov("Query finished! Start exporting files to SQL!", verbose)
+        export_to_sql(nodes, config["output_dir"], config["output_name"])
         echov(f"Exported results to: " + config["output_dir"])
 
     if not config["output_dir"] or verbose:
@@ -119,7 +142,9 @@ def search(context, search_type, query, **options):
             print("    " * node["depth"], f"           Title: {node['title']}")
             print(
                 "    " * node["depth"],
-                "           Related Videos: {}".format(node.get("relatedVideos")),
+                "           Related Videos: {}".format(
+                    node.get("relatedVideos")
+                ),
             )
 
 
@@ -144,17 +169,24 @@ def validate(config):
     if isinstance(config["number"], int):
         config["number"] = tuple((config["number"],))
 
+    # Force api_key to be a list of next index / key list.
+    if isinstance(config["api_key"], str):
+        config["keys"] = [0, [config["api_key"]]]
 
-def get_handle(api_key):
+    if isinstance(config["api_key"], list):
+        config["keys"] = [0, config["api_key"]]
+
+
+def get_handle(keys):
     """ Obtains the YouTube resource handle using an API key. """
     echov("Starting YouTube authentication.", verbose)
-    if not api_key:
+    if not keys:
         echoe(
             """You need to provide an API key using `--api-key`
         or the configuration file in order to query YouTube's API.
         Please see README on how to obtain such a key."""
         )
-    handle = get_youtube_handle(api_key)
+    handle = get_youtube_handle(keys)
     echov("API access established.", verbose)
     return handle
 
@@ -185,22 +217,45 @@ def build_nodes(config, handle, api_options, starter_videos):
         video.update({"rank": rank, "depth": 0})
     queue = deque(starter_videos)
     processed = []
+    processed_ids = set()
     while len(queue) > 0:
         video = queue.popleft()
         echov(
-            f"Processing video {video['videoId']} (Depth: {video['depth']}).", verbose
+            f"Processing video {video['videoId']} (Depth: {video['depth']}).",
+            verbose,
         )
         processed.append(video)
+        processed_ids.add(video["videoId"])
         if video["depth"] >= config["max_depth"]:
             video["relatedVideos"] = list()
             continue
         # Add children
         num_children = _get_branching(config["number"], video["depth"])
-        children = related_search(handle, num_children, video["videoId"], **api_options)
-        video["relatedVideos"] = list(map(lambda c: c["videoId"], children))
-        for rank, child in enumerate(children):
-            child.update({"rank": rank, "depth": video["depth"] + 1})
-        queue.extend(children)
+        while True:
+            try:
+                children = related_search(
+                    handle, num_children, video["videoId"], **api_options
+                )
+                video["relatedVideos"] = list(
+                    map(lambda c: c["videoId"], children)
+                )
+                for rank, child in enumerate(children):
+                    child.update({"rank": rank, "depth": video["depth"] + 1})
+                if config["unique"]:
+                    queue.extend(
+                        child
+                        for child in children
+                        if child["videoId"] not in processed_ids
+                    )
+                else:
+                    queue.extend(children)
+                break
+            except HttpError as e:
+                sys.tracebacklimit = 0
+                echow("Http error received:")
+                echow(e)
+                handle = get_youtube_handle(api_options["keys"])
+
     return processed
 
 
@@ -240,3 +295,5 @@ def _get_branching(container, index):
     clamped_index = min(maximal_index, index)
     clamped_index = max(minimal_index, clamped_index)
     return container[clamped_index]
+
+    # except HttpError as e:
