@@ -27,10 +27,12 @@ DEFAULT_OPTIONS = {
     "api_key": "",
     "output_dir": "",
     "output_format": "csv",
+    "output_name": "",
     "region_code": "de",
     "lang_code": "de",
     "safe_search": "none",
     "encoding": "utf-8",
+    "unique": False,
 }
 
 
@@ -93,7 +95,7 @@ def write_config(config, config_path=None):
         toml.dump(config, f)
 
 
-def update_config(config, options):
+def update_config(config, options={}):
     """ Updates a configuration dictionary and inserts default values.
 
         Parameters
